#!/usr/bin/env bash

syntax="syntax: $0 <source-file> | clean"
if [[ $1 = '' ]]; then
	echo $syntax
	exit -1
fi
if [[ $1 = 'clean' ]]; then
	rm -f multi.*
	rm -f *.out
	exit 0
fi
file=$1
if [ ! -f $file ]; then
	echo "Can't find file '$file'..."
	echo $syntax
	exit -1
fi

function ref() {
	compiler=$1
	$compiler -O0 $file && ./a.out > $compiler.0.out
	$compiler -O1 $file && ./a.out > $compiler.1.out
	$compiler -O2 $file && ./a.out > $compiler.2.out
	$compiler -O3 $file && ./a.out > $compiler.3.out
	if [ `diff $compiler.0.out $compiler.1.out` -o \
		  `diff $compiler.0.out $compiler.2.out` -o \
		  `diff $compiler.0.out $compiler.3.out` ]; then
		echo " -> Error: $compiler output differ between optimization levels"
		exit -2
	fi
	mv $compiler.0.out $compiler.out
	rm $compiler.?.out
	rm a.out
}

###################################################
echo " + Creating Reference Output"
ref gcc
ref clang
if [[ `diff gcc.out clang.out` ]]; then
	echo " -> Compilers differ in opinion"
	exit -3
fi

###################################################
echo " + Running Multi-Step (as test-suite would)"

clang -D_GNU_SOURCE -D__STDC_LIMIT_MACROS -DNDEBUG  -O3 -mllvm -disable-llvm-optzns  -m64 -fomit-frame-pointer -S $file -emit-llvm -o multi.ll > multi.log 2>&1
llvm-as multi.ll -o multi.bc >> multi.log 2>&1
opt -std-compile-opts -stats -time-passes multi.bc     -o multi.opt.bc >> multi.log 2>&1
opt -std-link-opts    -stats -time-passes multi.opt.bc -o multi.llvm.bc >> multi.log 2>&1
llc -asm-verbose=false -O3  multi.llvm.bc -o multi.llc.s -stats -time-passes >> multi.log 2>&1
clang multi.llc.s -o multi.llc  -lm   -m64 -fomit-frame-pointer -lm >> multi.log 2>&1

./multi.llc > multi.out

###################################################
echo " + Comparing the results"
if [[ `diff clang.out multi.out` ]]; then
	echo " -> Single/Multi differ in opinion"
	echo
	head *.out
	exit -4
else
	echo "All clear"
fi
